#!/usr/bin/env python
from __future__ import with_statement
import sys, time, pprint
import argparse
def main():
    parser = argparse.ArgumentParser(description="Diff two schema evolution files", prog="clixdiff", version='ClixDiff 0.4dev')
    parser.add_argument('left', type=argparse.FileType('r'), metavar='LEFT', help='left file for the diff')
    parser.add_argument('right', type=argparse.FileType('r'), metavar='RIGHT', help='right file for the diff')
    args = parser.parse_args()
    with args.left:
        whole_left = eval(args.left.read())
    with args.right:
        whole_right = eval(args.right.read())
    if whole_left['database'] != whole_right['database']:
        sys.stderr.write("Database names do not match -- aborting diff\n")
        sys.exit(1)
    if 'version' not in whole_left or whole_left['version'] < 2:
        sys.stderr.write("Unable to diff -- left file's version is unsupported!")
        sys.exit(1)
    if 'version' not in whole_right or whole_right['version'] < 2:
        sys.stderr.write("Unable to diff -- left file's version is unsupported!")
        sys.exit(1)
    if whole_left['version'] != whole_right['version']:
        sys.stderr.write("Unable to diff -- version mismatch")
        sys.exit(1)
    
    list_of_tables = []
    output_data = {'database': whole_left['database'], 'version': whole_left['version'], 'tables': list_of_tables}
    left_tables = whole_left['tables']
    right_tables = whole_right['tables']
    left_table_indexes = {}
    right_table_indexes = {}
    for i in range(len(left_tables)):
        left_table_indexes[left_tables[i]['name']] = i
    for i in range(len(right_tables)):
        right_table_indexes[right_tables[i]['name']] = i
    
    left_table_set = set(left_table_indexes)
    right_table_set = set(right_table_indexes)
    
    print`left_table_set`
    print`right_table_set`
    
    tables_to_drop = left_table_set - right_table_set
    print `tables_to_drop`
    for table_name in tables_to_drop:
        table = left_tables[left_table_indexes[table_name]]
        list_of_tables.append({'name': table['name'], 'action':'drop'})
    
    tables_to_create = right_table_set - left_table_set
    for table_name in tables_to_create:
        table = right_tables[right_table_indexes[table_name]]
        table['action'] = 'new'
        list_of_tables.append(table)
    
    tables_to_modify = left_table_set & right_table_set
    for table_name in tables_to_modify:
        ltable = left_tables[left_table_indexes[table_name]]
        rtable = right_tables[right_table_indexes[table_name]]
        merged_column_list = []
        merged_table = {'name': ltable['name'], 'action':'modify', 'columns':merged_column_list}
        left_columns = ltable['columns']
        right_columns = rtable['columns']
        left_column_indexes = {}
        right_column_indexes = {}
        for i in range(len(left_columns)):
            left_column_indexes[left_columns[i]['name']] = i
        for i in range(len(right_columns)):
            right_column_indexes[right_columns[i]['name']] = i
        
        left_column_set = set(left_column_indexes)
        right_column_set = set(right_column_indexes)
        
        columns_to_drop = left_column_set - right_column_set
        for column_name in columns_to_drop:
            merged_column_list.append({'name': column_name, 'action': 'drop'})
        
        columns_to_create = right_column_set - left_column_set
        for column_name in columns_to_create:
            column = right_columns[right_column_indexes[column_name]]
            column['action'] = 'new'
            merged_column_list.append(column)
        
        columns_to_modify = left_column_set & right_column_set
        for column_name in columns_to_modify:
            rcolumn = right_columns[right_column_indexes[column_name]]
            lcolumn = left_columns[left_column_indexes[column_name]]
            
            merged_column = {'name': rcolumn['name'], 'action': 'modify'}
            for k in rcolumn.keys():
                if k in ['name', 'action']: continue
                if lcolumn[k] == rcolumn[k]: continue
                merged_column[k] = rcolumn[k]
            if len(merged_column.keys()) > 2:
                merged_column_list.append(merged_column)
        if len(merged_column_list) > 0:
            list_of_tables.append(merged_table)
    
    # left = whole_left['tables']
    #     right = whole_right['tables']
    #     right_tables = {}
    #     for i in range(len(right)):
    #         right_tables[right[i]['name']] = i
    print "# AUTOGENERATED from clixdiff. Left: %s, Right: %s, Time: %s" % (args.left.name, args.right.name, time.ctime())
    print
    # tables_list = []
    #     output_data = {'database': whole_left['database'], 'version': 2, 'tables': tables_list}
    #     for table in left:
    #         if table['name'] not in right_tables.keys():
    #             #entire table has been deleted
    #             tables_list.append({'name':table['name'], 'action':'drop'})
    #         else:
    #             column_list = []
    #             table_output = {'name' : table['name'], 'columns':column_list}
    #             right_table = right[right_tables[table['name']]]
    #             right_columns = {}
    #             for i in range(len(right_table['columns'])):
    #                 right_columns[right_table['columns'][i]['name']] = i
    #             columns = table['columns']
    #             table_clean = True
    #             for c in columns:
    #                 if c['name'] not in right_columns:
    #                     column_list.append({'name':c['name'], 'action':'drop'})
    #                     table_clean = table_clean and False
    #                 else:
    #                     right_column = right_table['columns'][right_columns[c['name']]]
    #                     column_output = {'name':c['name'], 'action':'modify'}
    #                     column_clean = True
    #                     for prop in c.keys():
    #                         if prop == 'name': continue
    #                         if c[prop] == right_column[prop]: continue
    #                         column_output[prop] = right_column[prop]
    #                         column_clean = column_clean and False
    #                         table_clean = table_clean and False
    #                     if column_clean:
    #                         continue
    #                     column_list.append(column_output)
    #             left_columns = [i['name'] for i in columns]
    #             for i in right_columns:
    #                 if i in left_columns:
    #                     continue
    #                 column_list.append(right_table['columns'][right_columns[i]])
    #             column_list.sort(key=lambda i: i['name'])
    #             if table_clean: 
    #                 continue
    #             tables_list.append(table_output)
    #     left_tables = [i['name'] for i in left]
    #     leftover = set(right_tables.keys()) - set(left_tables)
    #     if len(leftover) > 0:
    #         for table in leftover:
    #             tables_list.append(right[right_tables[table]])
    #     tables_list.sort(key=lambda i: i['name'])
    pprint.pprint(output_data)

if __name__ == "__main__": main()